@@ -48,7 +48,10 @@ namespace rude{
 namespace config{
 
 enum SectionState{ STARTSECTION, SECTIONID, ESCAPEID, ENDSECTIONID, SECTIONCOMMENT, FOUNDIDONLY, FOUNDIDCOMMENT, SECTIONERROR, ENDSECTION };
-enum KeyValueState{ KEY, KEYESCAPE, STARTVALUE, COMMENT, FINDCOMMENT, KVERROR, ENDKV, VALUE, QUOTEVALUE, NONQUOTEVALUE, QUOTEESCAPE, NONQUOTEESCAPE, ENDKEYVALUE, LEFTARROW, RIGHTARROW, GETSOURCE, GETDEST, GETWIDTH, SDCOMMENT, ENDSOURCEDEST};
+/* remedius
+ * GETCOMMTYPE, GETPROCMETHOD were added in order to support runtime configuration options for choosing communication type and pre-/post-processing method
+ */
+enum KeyValueState{ KEY, KEYESCAPE, STARTVALUE, COMMENT, FINDCOMMENT, KVERROR, ENDKV, VALUE, QUOTEVALUE, NONQUOTEVALUE, QUOTEESCAPE, NONQUOTEESCAPE, ENDKEYVALUE, LEFTARROW, RIGHTARROW, GETSOURCE, GETDEST, GETWIDTH, GETCOMMTYPE, GETPROCMETHOD, SDCOMMENT, ENDSOURCEDEST};
 
 void ParserJuly2004::stripTrailing(std::string& buffer)
 {
@@ -150,12 +153,12 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							c = infile.peek();
 							if(c == EOF)
 							{
-								setError("102", "End of stream found before section ID");
+								setError("102", "End of stream found before section ID.");
 								sectionState = SECTIONERROR;
 							}
 							else if(isEOL(c))
 							{
-								setError("101", "End of line found before section ID");
+								setError("101", "End of line found before section ID.");
 								sectionState = SECTIONERROR;   
 							}
 							else if(c == ' ' || c == '\t')
@@ -185,12 +188,12 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							c = infile.peek();
 							if(c == EOF)
 							{
-								setError("104", "End of stream found before end-of-section marker");
+								setError("104", "End of stream found before end-of-section marker.");
 								sectionState = SECTIONERROR;
 							}
 							else if(isEOL(c))
 							{
-								setError("103", "End of line found before end-of-section marker");
+								setError("103", "End of line found before end-of-section marker.");
 								sectionState = SECTIONERROR;	     
 							}
 							else if(c == '\\')
@@ -226,12 +229,12 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							c = infile.peek();
 							if(c == EOF)
 							{
-								setError("107", "End of stream found after un-escaped backslash");
+								setError("107", "End of stream found after un-escaped backslash.");
 								sectionState = SECTIONERROR;
 							}
 							else if(isEOL(c))
 							{
-								setError("108", "Escaped new-line is not allowed in section ID or key");
+								setError("108", "Escaped new-line is not allowed in section ID or key.");
 								sectionState = SECTIONERROR;
 							}
 							else
@@ -266,7 +269,7 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							}
 							else
 							{
-								setError("105", "Illegal character found after end-of-section marker");
+								setError("105", "Illegal character found after end-of-section marker.");
 								sectionState = SECTIONERROR;
 							}
 							break;
@@ -353,6 +356,10 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
                         std::string destApp = "";
                         std::string destObj = "";
                         std::string width = "";
+                        /* remedius */
+                        std::string commType = "";
+                        /* remedius */
+                        std::string procMethod = "";
 
 			while (kvState != ENDKEYVALUE)
 			{						
@@ -483,12 +490,12 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							int c = infile.peek();
 							if(c == EOF)
 							{
-								setError("107", "End of stream found after un-escaped backslash");
+								setError("107", "End of stream found after un-escaped backslash.");
 								kvState = KVERROR;
 							}
 							else if(isEOL(c))
 							{
-								setError("108", "Escaped new-line is not allowed in key");
+								setError("108", "Escaped new-line is not allowed in key.");
 								kvState = KVERROR;
 							}
 							else
@@ -617,7 +624,7 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							int c = infile.peek();
 							if(c == EOF)
 							{
-								setError("106", "End of stream found before final quote (\") in value");
+								setError("106", "End of stream found before final quote (\") in value.");
 								kvState = KVERROR;
 							}
 							else if(c == '"')
@@ -653,7 +660,7 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							int c = infile.peek();
 							if(c == EOF)
 							{
-								setError("107", "End of stream found after un-escaped backslash");
+								setError("107", "End of stream found after un-escaped backslash.");
 								kvState = KVERROR;
 							}
 							else
@@ -712,7 +719,7 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 							int c = infile.peek();
 							if(c == EOF)
 							{
-								setError("107", "End of stream found after un-escaped backslash");
+								setError("107", "End of stream found after un-escaped backslash.");
 								kvState = KVERROR;
 							}
 							else
@@ -1012,12 +1019,22 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
 
                                                 if('0' <= c && c <= '9' && width.length() > 0)
                                                   {
-                                                    setError("902", "Two numbers detected in the width declaration");
+                                                    setError("902", "Two numbers detected in the width declaration.");
                                                     kvState = KVERROR;
                                                   }
 
                                                 // LOOP
                                               }
+                                            /* remedius
+                                             * communication type (GETCOMMTYPE) and processing method(GETPROCMETHOD) parameters are optional.
+                                             */
+                                            else if(c == ',')
+                                            {
+                                            	stripTrailing(width);
+                                            	infile.get(); // Throw away ,
+                                            	kvState = GETCOMMTYPE;
+
+                                            }
                                             else if(c == ']')
                                               {
                                                 stripTrailing(width);
@@ -1031,11 +1048,106 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
                                                 width += infile.get();
                                               }
                                             else {
-                                              setError("903", "Non integer number detected");
+                                              setError("903", "Non integer number detected.");
                                               kvState = KVERROR;
                                             }
                                             break;
                                           }
+                                          /* remedius */
+                                        case GETCOMMTYPE:
+                                        {
+                                            int c = infile.peek();
+
+                                            if(c == EOF || isEOL(c))
+                                              {
+                                                setError("901", "Partial communication type is found. No terminating ']'.");
+                                                kvState = KVERROR;
+                                              }
+                                            else if(d_commentchar != 0 && c == d_commentchar)
+                                              {
+                                                // discard '#'
+                                                //
+                                                infile.get();
+
+                                                kvState = SDCOMMENT;
+                                              }
+                                            else if(c == ' ' || c == '\t')
+                                              {
+                                                // discard whitespace
+                                                //
+                                                infile.get();
+
+                                                // LOOP
+                                              }
+                                            else if(c == ']')
+                                              {
+                                                stripTrailing(commType);
+                                                infile.get(); // Throw away ]
+                                                kvState = ENDSOURCEDEST;
+
+                                                // Ignoring any comments that might be here...
+                                              }
+                                            else if(c == ',')
+                                            {
+                                            	stripTrailing(commType);
+                                            	infile.get(); // Throw away ,
+                                            	kvState = GETPROCMETHOD;
+                                            }
+                                            else
+                                              {
+                                                commType += infile.get();
+                                              }
+
+                                            break;
+
+                                        }
+                                        /* remedius */
+                                        case GETPROCMETHOD:
+                                        {
+                                        	int c = infile.peek();
+
+                                        	if(c == EOF || isEOL(c))
+                                        	{
+                                        		setError("901", "Partial processing mathod is found. No terminating ']'.");
+                                        		kvState = KVERROR;
+                                        	}
+                                        	else if(d_commentchar != 0 && c == d_commentchar)
+                                        	{
+                                        		// discard '#'
+                                        		//
+                                        		infile.get();
+
+                                        		kvState = SDCOMMENT;
+                                        	}
+                                        	else if(c == ' ' || c == '\t')
+                                        	{
+                                        		// discard whitespace
+                                        		//
+                                        		infile.get();
+
+                                        		// LOOP
+                                        	}
+                                        	else if(c == ']')
+                                        	{
+                                        		stripTrailing(commType);
+                                        		infile.get(); // Throw away ]
+                                        		kvState = ENDSOURCEDEST;
+
+                                        		// Ignoring any comments that might be here...
+                                        	}
+                                        	else if(c != ',')
+                                        	{
+                                        		procMethod += infile.get();
+                                        	}
+                                        	else
+                                        	{
+                                        		setError("904", "Unexpected symbol.");
+                                        		kvState = KVERROR;
+                                        	}
+
+                                        	break;
+
+                                        }
                                         case SDCOMMENT:
                                           {
                                             //std::cout << "SDCOMMENT\n";
@@ -1068,8 +1180,12 @@ bool ParserJuly2004::parse(std::istream& infile, AbstractOrganiser& organiser)
                                             stripTrailing(destApp);
                                             stripTrailing(destObj);
                                             stripTrailing(width);
-
-                                            organiser.foundSourceDest(srcApp.c_str(), srcObj.c_str(), destApp.c_str(), destObj.c_str(), width.c_str(), comment.c_str());
+                                            /* remedius */
+                                            stripTrailing(commType);
+                                            /* remedius */
+                                            stripTrailing(procMethod);
+                                            /* remedius */
+                                            organiser.foundSourceDest(srcApp.c_str(), srcObj.c_str(), destApp.c_str(), destObj.c_str(), width.c_str(), commType.c_str(), procMethod.c_str(), comment.c_str());
                                             
                                             kvState = ENDKEYVALUE;
                                             break;
