@@ -236,6 +236,12 @@ public:
         const char *getSrcAppAt(int index) const;
         const char *getDestAppAt(int index) const;
         const char *getWidthAt(int index) const;
+        /* remedius
+         */
+        const char *getCommTypeAt(int index) const;
+        /* remedius
+         */
+        const char *getProcMethodAt(int index) const;
 
 	//=
 	// Returns the number of sections in the entire configuration file
