@@ -19,6 +19,7 @@
 // This header file needs to be included first since array_data.hh
 // includes <mpi.h> which musy be included first on BG/L
 #include "music/array_data.hh"
+#if MUSIC_USE_MPI
 
 #include "music/linear_index.hh"
 
@@ -51,3 +52,4 @@ namespace MUSIC {
   }
   
 }
+#endif
