@@ -15,12 +15,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+#include "music/port.hh"
 
-//#define MUSIC_DEBUG 1
-#include "music/debug.hh"
+#if MUSIC_USE_MPI
+#include "music/event_router.hh"
 
 #include "music/setup.hh" // Must be included first on BG/L
-#include "music/port.hh"
 #include "music/error.hh"
 
 namespace MUSIC {
@@ -28,6 +28,7 @@ namespace MUSIC {
   Port::Port (Setup* s, std::string identifier)
     : portName_ (identifier), setup_ (s), isMapped_ (false)
   {
+    s->maybePostponedSetup ();
     ConnectivityInfo_ = s->portConnectivity (portName_);
     setup_->addPort (this);
   }
@@ -112,20 +113,37 @@ namespace MUSIC {
 
 
   void
-  OutputRedistributionPort::setupCleanup ()
+  Port::checkIndexMap (IndexMap* indexMap)
   {
-    // NOTE: Cleanup resources only used during setup phase,
-    if (spatialNegotiator)
-      delete spatialNegotiator;
+    if (indexMap->begin () != indexMap->end ())
+      {
+	IndexMap::iterator i = indexMap->begin ();
+	IndexInterval last = *i;
+	++i;
+	for (; i != indexMap->end (); ++i)
+	  {
+	    if (i->begin () < last.end ())
+	      {
+		std::ostringstream os;
+		os << "Mapping port " << portName_
+		   << " using index map with overlaps: ["
+		   << last.begin () << ", " << last.end () << "), ["
+		   << i->begin () << ", " << i->end () << ")";
+		error (os);
+	      }
+	    last = *i;
+	  }
+      }
   }
-  
+
 
   void
-  OutputRedistributionPort::mapImpl (IndexMap* indices,
-				     Index::Type type,
-				     int maxBuffered,
-				     int dataSize)
+  OutputPort::mapImpl (IndexMap* indices,
+		       Index::Type type,
+		       int maxBuffered,
+		       int dataSize)
   {
+    checkIndexMap (indices);
     // The timing algorithm in uses a different definition of
     // maxBuffered.  While the interface in port.hh counts the number
     // of "ticks of data" which must be stored, the timing algorithm
@@ -137,13 +155,14 @@ namespace MUSIC {
     // Retrieve info about all remote connectors of this port
     PortConnectorInfo portConnections
       = ConnectivityInfo_->connections ();
-    spatialNegotiator = new SpatialOutputNegotiator (indices, type);
+    indices_ = indices;
+    index_type_ = type;
     for (PortConnectorInfo::iterator info = portConnections.begin ();
 	 info != portConnections.end ();
 	 ++info)
       {
 	// Create connector
-	OutputConnector* connector = makeOutputConnector (*info);
+	Connector* connector = makeConnector (*info);
 	setup_->addConnection (new OutputConnection (connector,
 						     maxBuffered,
 						     dataSize));
@@ -151,22 +170,16 @@ namespace MUSIC {
   }
 
 
-  void
-  InputRedistributionPort::setupCleanup ()
-  {
-    // NOTE: Cleanup resources only used during setup phase,
-    if (spatialNegotiator)
-      delete spatialNegotiator;
-  }
 
   
   void
-  InputRedistributionPort::mapImpl (IndexMap* indices,
-				    Index::Type type,
-				    double accLatency,
-				    int maxBuffered,
-				    bool interpolate)
+  InputPort::mapImpl (IndexMap* indices,
+		      Index::Type type,
+		      double accLatency,
+		      int maxBuffered,
+		      bool interpolate)
   {
+    checkIndexMap (indices);
     // The timing algorithm in uses a different definition of
     // maxBuffered.  While the interface in port.hh counts the number
     // of "ticks of data" which must be stored, the timing algorithm
@@ -179,8 +192,9 @@ namespace MUSIC {
     PortConnectorInfo portConnections
       = ConnectivityInfo_->connections ();
     PortConnectorInfo::iterator info = portConnections.begin ();
-    spatialNegotiator = new SpatialInputNegotiator (indices, type);
-    InputConnector* connector = makeInputConnector (*info);
+    indices_ = indices;
+    index_type_ = type;
+    Connector* connector = makeConnector (*info);
     ClockState integerLatency (accLatency, setup_->timebase ());
     setup_->addConnection (new InputConnection (connector,
 						maxBuffered,
@@ -222,21 +236,34 @@ namespace MUSIC {
   {
     sampler.configure (dmap);
     type_ = dmap->type ();
-    OutputRedistributionPort::mapImpl (dmap->indexMap (),
+    OutputPort::mapImpl (dmap->indexMap (),
 				       Index::GLOBAL,
 				       maxBuffered,
 				       0);
   }
 
 
-  OutputConnector*
-  ContOutputPort::makeOutputConnector (ConnectorInfo connInfo)
+  Connector*
+  ContOutputPort::makeConnector (ConnectorInfo connInfo)
   {
-    return new ContOutputConnector (connInfo,
-				    spatialNegotiator,
-				    setup_->communicator (),
-				    sampler,
-				    type_);
+	  Connector * conn;
+	  if(connInfo.communicationType() ==  ConnectorInfo::POINTTOPOINT){
+		  conn =  new ContOutputConnector (connInfo,
+				  indices_,
+		  		  index_type_,
+				  setup_->communicator (),
+				  sampler,
+				  type_);
+	  }
+	  else
+		  conn = new ContOutputCollectiveConnector(connInfo,
+				  indices_,
+		  		  index_type_,
+				  setup_->communicator (),
+				  sampler,
+				  type_);
+
+	  return conn;
   }
   
   
@@ -295,7 +322,7 @@ namespace MUSIC {
     sampler.configure (dmap);
     delay_ = delay;
     type_ = dmap->type ();
-    InputRedistributionPort::mapImpl (dmap->indexMap (),
+    InputPort::mapImpl (dmap->indexMap (),
 				      Index::GLOBAL,
 				      delay,
 				      maxBuffered,
@@ -303,15 +330,28 @@ namespace MUSIC {
   }
 
   
-  InputConnector*
-  ContInputPort::makeInputConnector (ConnectorInfo connInfo)
+  Connector*
+  ContInputPort::makeConnector (ConnectorInfo connInfo)
   {
-    return new ContInputConnector (connInfo,
-				   spatialNegotiator,
-				   setup_->communicator (),
-				   sampler,
-				   type_,
-				   delay_);
+	  Connector * conn;
+	  if(connInfo.communicationType() ==  ConnectorInfo::POINTTOPOINT){
+		  conn = new ContInputConnector (connInfo,
+				  indices_,
+		  		  index_type_,
+				  setup_->communicator (),
+				  sampler,
+				  type_,
+				  delay_);
+	  }
+	  else
+		  conn = new ContInputCollectiveConnector (connInfo,
+				  indices_,
+		  		  index_type_,
+				  setup_->communicator (),
+				  sampler,
+				  type_,
+				  delay_);
+	  return conn;
   }
 
   
@@ -320,19 +360,73 @@ namespace MUSIC {
    * Event Ports
    *
    ********************************************************************/
-  
+
   EventOutputPort::EventOutputPort (Setup* s, std::string id)
-    : Port (s, id), routingMap (new EventRoutingMap ())
+    : Port (s, id), routingMap (new OutputRoutingMap () /* deleted in buildTable */)
   {
+    /* remedius
+     * Depending on the communication type (<commType>) and
+     * processing method (<procMethod>) that was introduced as
+     * runtime configuration options,
+     * particular processing router should be created on the output side.
+     * When collective communication type is used,
+     * then the processing method has to be TABLE on the output side, as in this case
+     * the processing on the output side means just an insertion of the event to the buffer.
+     * The difference between point-to-point and collective communication types is that on the output side
+     * in the latest case we have only one common buffer (one CollectiveSubconnector) and processing is happening on the receiver side.
+     */
+    if (isConnected ())
+      {
+	bool mixed = false;
+	PortConnectorInfo::iterator c = ConnectivityInfo_->connections ().begin ();
+	int commType = c->communicationType ();
+	int procMethod = c->processingMethod ();
+	for (++c; c != ConnectivityInfo_->connections ().end (); ++c)
+	  {
+	    if (c->processingMethod () != procMethod)
+	      error ("MUSIC: can't use both tree and table for same port");
+	    if (c->communicationType () != commType)
+	      mixed = true;
+	  }
+	if (!mixed)
+	  {
+	    if (commType == ConnectorInfo::COLLECTIVE)
+	      router = new DirectRouter ();
+	    else if (procMethod == ConnectorInfo::TREE)
+	      router = new TreeProcessingOutputRouter ();
+	    else
+	      router = new TableProcessingOutputRouter ();
+	  }
+	else
+	  {
+	    if (procMethod == ConnectorInfo::TREE)
+	      router = new HybridTreeProcessingOutputRouter ();
+	    else
+	      router = new HybridTableProcessingOutputRouter ();
+	  }
+      }
   }
 
+
+  EventOutputPort::~EventOutputPort()
+ {
+   if (router != NULL)
+     delete router;
+ }
   
+  void 
+  EventOutputPort::mapImpl (IndexMap* indices,
+			    Index::Type type,
+			    int maxBuffered)
+  {
+    OutputPort::mapImpl (indices, type, maxBuffered, (int) sizeof (Event));
+  }
+
   void
   EventOutputPort::map (IndexMap* indices, Index::Type type)
   {
     assertOutput ();
-    int maxBuffered = MAX_BUFFERED_NO_VALUE;
-    mapImpl (indices, type, maxBuffered, sizeof (Event));
+    EventOutputPort::mapImpl (indices, type, MAX_BUFFERED_NO_VALUE);
   }
 
   
@@ -346,49 +440,68 @@ namespace MUSIC {
       {
 	error ("EventOutputPort::map: maxBuffered should be a positive integer");
       }
-    mapImpl (indices, type, maxBuffered, sizeof (Event));
+    EventOutputPort::mapImpl (indices, type, maxBuffered);
   }
 
   
-  OutputConnector*
-  EventOutputPort::makeOutputConnector (ConnectorInfo connInfo)
+  Connector*
+  EventOutputPort::makeConnector (ConnectorInfo connInfo)
   {
-    return new EventOutputConnector (connInfo,
-				     spatialNegotiator,
-				     setup_->communicator (),
-				     routingMap);
+    Connector *conn;
+    // we need to choose a right connector according to the communication type
+    if (connInfo.communicationType () ==  ConnectorInfo::POINTTOPOINT)
+      conn = new EventOutputConnector (connInfo,
+				       indices_,
+				       index_type_,
+				       setup_->communicator (),
+				       routingMap);
+    else
+      conn = new EventOutputCollectiveConnector
+	(connInfo,
+	 indices_,
+	 index_type_,
+	 setup_->communicator (),
+	 router->directRouter ());
+
+    return conn;
   }
   
   
   void
-  EventOutputPort::buildTable ()
+  EventOutputPort::insertEventImpl (double t, int id)
   {
-    routingMap->fillRouter (router);
-    delete routingMap;
-    router.buildTable ();
+    router->processEvent(t, id);
   }
 
-  
   void
   EventOutputPort::insertEvent (double t, GlobalIndex id)
   {
-    router.insertEvent (t, id);
+    insertEventImpl(t, id);
   }
 
   
   void
   EventOutputPort::insertEvent (double t, LocalIndex id)
   {
-    router.insertEvent (t, id);
+    insertEventImpl(t, id);
   }
 
-  
+  void
+  EventOutputPort::buildTable ()
+  {
+    routingMap->fillRouter (router);
+    delete routingMap;
+    router->buildTable ();
+  }
+
+
   EventInputPort::EventInputPort (Setup* s, std::string id)
     : Port (s, id)
   {
-  }
 
+  }
   
+
   void
   EventInputPort::map (IndexMap* indices,
 		       EventHandlerGlobalIndex* handleEvent,
@@ -466,7 +579,7 @@ namespace MUSIC {
   {
     type_ = type;
     handleEvent_ = handleEvent;
-    InputRedistributionPort::mapImpl (indices,
+    InputPort::mapImpl (indices,
 				      type,
 				      accLatency,
 				      maxBuffered,
@@ -474,14 +587,25 @@ namespace MUSIC {
   }
 
   
-  InputConnector*
-  EventInputPort::makeInputConnector (ConnectorInfo connInfo)
+  Connector*
+  EventInputPort::makeConnector (ConnectorInfo connInfo)
   {
-    return new EventInputConnector (connInfo,
-				    spatialNegotiator,
-				    handleEvent_,
-				    type_,
-				    setup_->communicator ());
+	  // we need to choose a right connector according to the communication type
+	  Connector *conn;
+	  if(connInfo.communicationType() ==  ConnectorInfo::POINTTOPOINT)
+		  conn =   new EventInputConnector (connInfo,
+		  			  indices_,
+		  			  index_type_,
+		  			  setup_->communicator (),
+		  			  handleEvent_);
+	  else
+		  conn = new EventInputCollectiveConnector(connInfo,
+	  	 			indices_,
+		  			index_type_,
+	  	 			setup_->communicator (),
+	  	 			handleEvent_);
+
+	 return conn;
   }
 
   
@@ -499,7 +623,6 @@ namespace MUSIC {
     cEventHandlerLocalIndex = EventHandlerLocalIndexProxy (eh);
     return &cEventHandlerLocalIndex;
   }
-  
 
   /********************************************************************
    *
@@ -545,18 +668,19 @@ namespace MUSIC {
   {
     // Identify ourselves
     LinearIndex indices (rank_, 1);
-    OutputRedistributionPort::mapImpl (&indices,
+    OutputPort::mapImpl (&indices,
 				       Index::GLOBAL,
 				       maxBuffered,
 				       1);
   }
   
   
-  OutputConnector*
-  MessageOutputPort::makeOutputConnector (ConnectorInfo connInfo)
+  Connector*
+  MessageOutputPort::makeConnector (ConnectorInfo connInfo)
   {
     return new MessageOutputConnector (connInfo,
-				       spatialNegotiator,
+				       indices_,
+				       index_type_,
 				       setup_->communicator (),
 				       buffers);
   }
@@ -664,7 +788,7 @@ namespace MUSIC {
     handleMessage_ = handleMessage;
     // Receive from everybody
     LinearIndex indices (0, handleMessage ? Index::WILDCARD_MAX : 0);
-    InputRedistributionPort::mapImpl (&indices,
+    InputPort::mapImpl (&indices,
 				      Index::GLOBAL,
 				      accLatency,
 				      maxBuffered,
@@ -672,13 +796,13 @@ namespace MUSIC {
   }
 
   
-  InputConnector*
-  MessageInputPort::makeInputConnector (ConnectorInfo connInfo)
+  Connector*
+  MessageInputPort::makeConnector (ConnectorInfo connInfo)
   {
     return new MessageInputConnector (connInfo,
-				      spatialNegotiator,
+				      indices_,
+		  			  index_type_,
 				      handleMessage_,
-				      Index::GLOBAL,
 				      setup_->communicator ());
   }
 
@@ -694,3 +818,4 @@ namespace MUSIC {
 
   
 }
+#endif
